
import java.applet.Applet;
import java.awt.*;  // for Graphics, Color, Image, Point, ...
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
//import java.lang.Math;
import java.util.Vector;
import java.awt.geom.Path2D;

public class SimpleRasterPaint extends Applet implements
	KeyListener,
	MouseListener,
	MouseMotionListener
{
	int appletWidth, appletHeight;
	int canvasWidth, canvasHeight;

	Image backbuffer;
	Graphics2D backg;

	MyButton button;
	boolean isButtonActive = false;
	private static final int buttonHeight = 30;

	float currentAlpha = 0.3f; // in the interval [0,1]
	float currentBrushRadius = 10; // in pixels
	Color currentColor = new Color( 0.0f, 0.0f, 0.0f, currentAlpha );

	int mouse_x, mouse_y;

	boolean isBrushStrokeBeingDrawn = false;

	public void init() {
		appletWidth = getSize().width;
		appletHeight = getSize().height;
		canvasWidth = appletWidth;
		canvasHeight = appletHeight - buttonHeight;

		backbuffer = createImage( canvasWidth, canvasHeight );
		backg = (Graphics2D)backbuffer.getGraphics();
		backg.setColor( Color.white );
		backg.fillRect( 0, 0, canvasWidth, canvasHeight );

		addKeyListener( this );
		addMouseListener( this );
		addMouseMotionListener( this );

		button = new MyButton( 0, appletHeight - buttonHeight, canvasWidth, buttonHeight, "Clear" );
	}

	public void keyPressed( KeyEvent e ) { }
	public void keyReleased( KeyEvent e ) { }
	public void keyTyped( KeyEvent e ) {
		char c = e.getKeyChar();
		System.out.println("key typed: "+c);
		if ( c != KeyEvent.CHAR_UNDEFINED ) {
			//...
			repaint();
			e.consume();
		}
	}

	public void mouseEntered( MouseEvent e ) { }
	public void mouseExited( MouseEvent e ) { }
	public void mouseClicked( MouseEvent e ) {
		e.consume();
	}
	public void mousePressed( MouseEvent e ) {
		mouse_x = e.getX();
		mouse_y = e.getY();
		if ( button.isMouseOverWidget( e.getX(), e.getY() ) ) {
			isButtonActive = true;
			button.pressEvent( e.getX(), e.getY() );
		}
		else {
			isBrushStrokeBeingDrawn = true;

			backg.setColor( currentColor );
			int r = Math.round( currentBrushRadius );
			backg.fillOval( mouse_x - r/2, mouse_y - r/2, r, r );
		}
		repaint();
	}
	public void mouseReleased( MouseEvent e ) {
		mouse_x = e.getX();
		mouse_y = e.getY();
		if ( isButtonActive ) {
			if ( button.isMouseOverWidget( e.getX(), e.getY() ) ) {
				backg.setColor( Color.white );
				backg.fillRect( 0, 0, canvasWidth, canvasHeight );
			}
			button.releaseEvent( e.getX(), e.getY() );
			isButtonActive = false;
		}
		else if ( isBrushStrokeBeingDrawn ) {
			isBrushStrokeBeingDrawn = false;
		}

		currentColor = new Color( 0.0f, 0.0f, 0.0f, currentAlpha );

		repaint();

	}
	public void mouseMoved( MouseEvent e ) {
		mouse_x = e.getX();
		mouse_y = e.getY();
		repaint();
	}
	public void mouseDragged( MouseEvent e ) {
		mouse_x = e.getX();
		mouse_y = e.getY();
		repaint();
		e.consume();
	}

	public void update( Graphics g ) {
		g.drawImage( backbuffer, 0, 0, this );
		Graphics2D g2 = (Graphics2D)g;

		// draw mouse cursor
		final int cursorRadius = 20;
		g2.setColor( Color.red );
		g2.drawLine( mouse_x - cursorRadius, mouse_y, mouse_x + cursorRadius, mouse_y );
		g2.drawLine( mouse_x, mouse_y - cursorRadius, mouse_x, mouse_y + cursorRadius );
		g2.drawOval( mouse_x - cursorRadius, mouse_y - cursorRadius, 2*cursorRadius, 2*cursorRadius );

		button.draw( g2 );
	}

	public void paint( Graphics g ) {
		update( g );
	}

	public String getAppletInfo() {
		return "Écrit par M. Astérix";
	}
}


